@@ -50,6 +50,7 @@ import java.text.SimpleDateFormat;
 
 import java.util.ArrayList;
 import java.util.Calendar;
+import java.util.Locale;
 import java.util.TimeZone;
 
 
@@ -71,7 +72,7 @@ import java.util.TimeZone;
  * </p>
  *
  * @author Mark Matthews
- * @version $Id: PreparedStatement.java,v 1.27.2.45 2004/08/27 21:50:12 mmatthew Exp $
+ * @version $Id: PreparedStatement.java,v 1.27.2.51 2005/04/07 23:55:57 mmatthews Exp $
  *
  * @see java.sql.ResultSet
  * @see java.sql.PreparedStatement
@@ -97,6 +98,24 @@ public class PreparedStatement extends com.mysql.jdbc.Statement
     private boolean useTrueBoolean = false;
     private char firstCharOfStmt = 0;
 
+    private final static byte[] HEX_DIGITS = new byte[] {(byte)'0',
+    	(byte)'1',
+    	(byte)'2',
+    	(byte)'3',
+    	(byte)'4',
+    	(byte)'5',
+    	(byte)'6',
+    	(byte)'7',
+    	(byte)'8',
+    	(byte)'9',
+    	(byte)'A',
+    	(byte)'B',
+    	(byte)'C',
+    	(byte)'D',
+    	(byte)'E',
+    	(byte)'F',
+    };
+    
     /**
      * Constructor for the PreparedStatement class.
      *
@@ -213,7 +232,7 @@ public class PreparedStatement extends com.mysql.jdbc.Statement
         if (x == null) {
             setNull(parameterIndex, java.sql.Types.DECIMAL);
         } else {
-            setInternal(parameterIndex, fixDecimalExponent(x.toString()));
+            setInternal(parameterIndex, StringUtils.fixDecimalExponent(x.toString()));
         }
     }
 
@@ -310,14 +329,43 @@ public class PreparedStatement extends com.mysql.jdbc.Statement
      */
     public void setBytes(int parameterIndex, byte[] x)
     	throws SQLException {
-    	setBytes(parameterIndex, x, true);
+    	setBytes(parameterIndex, x, true, true);
     }
     
-    protected void setBytes(int parameterIndex, byte[] x, boolean checkForIntroducer)
+    protected void setBytes(int parameterIndex, byte[] x, boolean checkForIntroducer, boolean escapeForMBChars)
         throws SQLException {
         if (x == null) {
             setNull(parameterIndex, java.sql.Types.BINARY);
         } else {
+        	String connectionEncoding = this.connection.getEncoding();
+        	
+        	if (escapeForMBChars &&
+        		this.connection.useUnicode() &&
+        		connectionEncoding != null &&
+        		CharsetMapping.MULTIBYTE_CHARSETS.get(
+        				connectionEncoding.toLowerCase(Locale.ENGLISH)) != null) {
+        		
+        		// Send as hex
+        		
+        		ByteArrayOutputStream bOut = new ByteArrayOutputStream((x.length * 2) + 3);
+        		bOut.write('x');
+        		bOut.write('\'');
+        		
+        		for (int i = 0; i < x.length; i++) {
+        			int lowBits = (x[i] & 0xff) / 16;
+        			int highBits = (x[i] & 0xff) % 16;
+        			
+        			bOut.write(HEX_DIGITS[lowBits]);
+        			bOut.write(HEX_DIGITS[highBits]);
+        		}
+        		
+        		bOut.write('\'');
+        		
+        		setInternal(parameterIndex, bOut.toByteArray());
+        		
+        		return;
+        	}
+        	
             // escape them
             int numBytes = x.length;
             
@@ -513,7 +561,7 @@ public class PreparedStatement extends com.mysql.jdbc.Statement
      */
     public void setDouble(int parameterIndex, double x)
         throws SQLException {
-        setInternal(parameterIndex, fixDecimalExponent(String.valueOf(x)));
+        setInternal(parameterIndex, StringUtils.fixDecimalExponent(String.valueOf(x)));
     }
 
     /**
@@ -526,7 +574,7 @@ public class PreparedStatement extends com.mysql.jdbc.Statement
      * @throws SQLException if a database access error occurs
      */
     public void setFloat(int parameterIndex, float x) throws SQLException {
-        setInternal(parameterIndex, fixDecimalExponent(String.valueOf(x)));
+        setInternal(parameterIndex, StringUtils.fixDecimalExponent(String.valueOf(x)));
     }
 
     /* (non-Javadoc)
@@ -1180,7 +1228,7 @@ public class PreparedStatement extends com.mysql.jdbc.Statement
     public void setTimestamp(int parameterIndex, Timestamp x)
         throws SQLException {
         setTimestampInternal(parameterIndex, x,
-            this.connection.getDefaultTimeZone());
+            this.connection.getDefaultTimeZone(), false);
     }
 
     /**
@@ -1195,7 +1243,7 @@ public class PreparedStatement extends com.mysql.jdbc.Statement
      */
     public void setTimestamp(int parameterIndex, java.sql.Timestamp x,
         Calendar cal) throws SQLException {
-        setTimestampInternal(parameterIndex, x, cal.getTimeZone());
+        setTimestampInternal(parameterIndex, x, cal.getTimeZone(), true);
     }
 
     /**
@@ -1694,6 +1742,11 @@ public class PreparedStatement extends com.mysql.jdbc.Statement
 
         setInternal(parameterIndex, parameterWithQuotes);
     }
+    
+    protected void setBytesNoEscapeNoQuotes(int parameterIndex,
+            byte[] parameterAsBytes) throws SQLException {
+            setInternal(parameterIndex, parameterAsBytes);
+    }
 
     /**
      * Sets wheather or not this statement should retreive generated keys.
@@ -2145,13 +2198,13 @@ public class PreparedStatement extends com.mysql.jdbc.Statement
      * @throws SQLException if a database-access error occurs.
      */
     private synchronized void setTimestampInternal(int parameterIndex,
-        Timestamp x, TimeZone tz) throws SQLException {
+        Timestamp x, TimeZone tz, boolean rollForward) throws SQLException {
         if (x == null) {
             setNull(parameterIndex, java.sql.Types.TIMESTAMP);
         } else {
             String timestampString = null;
             x = TimeUtil.changeTimezone(this.connection, x, tz,
-                    this.connection.getServerTimezone());
+                    this.connection.getServerTimezone(), rollForward);
 
             if (this.tsdf == null) {
                 this.tsdf = new SimpleDateFormat("''yyyy-MM-dd HH:mm:ss''");
@@ -2200,6 +2253,25 @@ public class PreparedStatement extends com.mysql.jdbc.Statement
         }
     }
 
+    /**
+     * Used to escape binary data with hex for mb charsets
+     * @param buf
+     * @param packet
+     * @param size
+     * @throws SQLException
+     */
+    private final void hexEscapeBlock(byte[] buf, Buffer packet, int size)
+    throws SQLException {
+    	for (int i = 0; i < size; i++) {
+            byte b = buf[i];
+            int lowBits = (b & 0xff) / 16;
+			int highBits = (b & 0xff) % 16;
+			
+			packet.writeByte(HEX_DIGITS[lowBits]);
+			packet.writeByte(HEX_DIGITS[highBits]);
+    	}
+    }
+
     private final void escapeblockFast(byte[] buf, Buffer packet, int size)
         throws SQLException {
         int lastwritten = 0;
@@ -2313,34 +2385,6 @@ public class PreparedStatement extends com.mysql.jdbc.Statement
         return sendPacket;
     }
 
-    //
-    // Adds '+' to decimal numbers that are positive (MySQL doesn't
-    // understand them otherwise
-    //
-    private static final String fixDecimalExponent(String dString) {
-        int ePos = dString.indexOf("E");
-
-        if (ePos == -1) {
-            ePos = dString.indexOf("e");
-        }
-
-        if (ePos != -1) {
-            if (dString.length() > (ePos + 1)) {
-                char maybeMinusChar = dString.charAt(ePos + 1);
-
-                if (maybeMinusChar != '-') {
-                    StringBuffer buf = new StringBuffer(dString.length() + 1);
-                    buf.append(dString.substring(0, ePos + 1));
-                    buf.append('+');
-                    buf.append(dString.substring(ePos + 1, dString.length()));
-                    dString = buf.toString();
-                }
-            }
-        }
-
-        return dString;
-    }
-
     private void initializeFromParseInfo() throws SQLException {
         this.staticSqlStrings = this.parseInfo.staticSql;
         this.hasLimitClause = this.parseInfo.foundLimitClause;
@@ -2476,6 +2520,18 @@ public class PreparedStatement extends com.mysql.jdbc.Statement
         boolean escape, int streamLength, boolean useLength)
         throws SQLException {
         try {
+        	String connectionEncoding = this.connection.getEncoding();
+        	
+        	boolean hexEscape = false;
+        	
+        	if (this.connection.useUnicode() &&
+        		connectionEncoding != null &&
+        		CharsetMapping.MULTIBYTE_CHARSETS.get(
+        				connectionEncoding.toLowerCase(Locale.ENGLISH)) != null &&
+        		!this.connection.parserKnowsUnicode()) {
+        		hexEscape = true;
+        	}
+        	
             if (streamLength == -1) {
                 useLength = false;
             }
@@ -2490,7 +2546,9 @@ public class PreparedStatement extends com.mysql.jdbc.Statement
 
             int lengthLeftToRead = streamLength - bc;
 
-            if (this.connection.getIO().versionMeetsMinimum(4, 1, 0)) {
+            if (hexEscape) {
+            	packet.writeStringNoNull("x");
+            } else if (this.connection.getIO().versionMeetsMinimum(4, 1, 0)) {
             	packet.writeStringNoNull("_binary");
             }
             
@@ -2499,7 +2557,9 @@ public class PreparedStatement extends com.mysql.jdbc.Statement
             }
 
             while (bc > 0) {
-                if (escape) {
+            	if (hexEscape) {
+            		hexEscapeBlock(streamConvertBuf, packet, bc);
+            	} else if (escape) {
                     escapeblockFast(streamConvertBuf, packet, bc);
                 } else {
                     packet.writeBytesNoNull(streamConvertBuf, 0, bc);
@@ -2617,6 +2677,8 @@ public class PreparedStatement extends com.mysql.jdbc.Statement
 
             statementLength = sql.length();
 
+            char[] sqlAsChars = sql.toCharArray();
+            
             ArrayList endpointList = new ArrayList();
             boolean inQuotes = false;
             boolean inQuotedId = false;
@@ -2632,7 +2694,7 @@ public class PreparedStatement extends com.mysql.jdbc.Statement
             foundLimitClause = false;
 
             for (i = 0; i < statementLength; ++i) {
-                char c = sql.charAt(i);
+                char c = sqlAsChars[i];
 
                 if ((firstStmtChar == 0) && !Character.isWhitespace(c)) {
                     // Determine what kind of statement we're doing (_S_elect, _I_nsert, etc.)
@@ -2666,16 +2728,16 @@ public class PreparedStatement extends com.mysql.jdbc.Statement
 
                 if (!inQuotes && (i < stopLookingForLimitClause)) {
                     if ((c == 'L') || (c == 'l')) {
-                        char posI1 = sql.charAt(i + 1);
+                        char posI1 = sqlAsChars[i + 1];
 
                         if ((posI1 == 'I') || (posI1 == 'i')) {
-                            char posM = sql.charAt(i + 2);
+                            char posM = sqlAsChars[i + 2];
 
                             if ((posM == 'M') || (posM == 'm')) {
-                                char posI2 = sql.charAt(i + 3);
+                                char posI2 = sqlAsChars[i + 3];
 
                                 if ((posI2 == 'I') || (posI2 == 'i')) {
-                                    char posT = sql.charAt(i + 4);
+                                    char posT = sqlAsChars[i + 4];
 
                                     if ((posT == 'T') || (posT == 't')) {
                                         foundLimitClause = true;
@@ -2710,13 +2772,13 @@ public class PreparedStatement extends com.mysql.jdbc.Statement
                 int len = end - begin;
 
                 if (foundLoadData) {
-                    String temp = new String(sql.toCharArray(), begin, len);
+                    String temp = new String(sqlAsChars, begin, len);
                     staticSql[i] = temp.getBytes();
                 } else if (encoding == null) {
                     byte[] buf = new byte[len];
 
                     for (int j = 0; j < len; j++) {
-                        buf[j] = (byte) sql.charAt(begin + j);
+                        buf[j] = (byte) sqlAsChars[begin + j];
                     }
 
                     staticSql[i] = buf;
@@ -2728,10 +2790,11 @@ public class PreparedStatement extends com.mysql.jdbc.Statement
 									begin, len,
                                     connection.parserKnowsUnicode());
                         } else {
-                            String temp = new String(sql.toCharArray(), begin,
+                            String temp = new String(sqlAsChars, begin,
                                     len);
-                            staticSql[i] = StringUtils.getBytes(temp, null,
-                                    encoding, connection.parserKnowsUnicode());
+                            staticSql[i] = StringUtils.getBytes(temp, encoding,
+                            		connection.getServerCharacterEncoding(), 
+                            		connection.parserKnowsUnicode());
                         }
                     } catch (java.io.UnsupportedEncodingException ue) {
                         throw new SQLException(ue.toString());
