@@ -50,13 +50,13 @@ import com.mysql.jdbc.SQLError;
  *
  * @author Mark Matthews
  * 
- * @version $Id: PreparedStatementWrapper.java,v 1.1.2.4 2004/08/09 22:15:12 mmatthew Exp $
+ * @version $Id: PreparedStatementWrapper.java,v 1.1.2.6 2004/12/15 21:30:45 mmatthew Exp $
  */
 class PreparedStatementWrapper extends StatementWrapper
     implements PreparedStatement {
-    PreparedStatementWrapper(MysqlPooledConnection conn,
+    PreparedStatementWrapper(ConnectionWrapper c, MysqlPooledConnection conn,
         PreparedStatement toWrap) {
-        super(conn, toWrap);
+        super(c, conn, toWrap);
     }
 
     /* (non-Javadoc)
@@ -683,7 +683,10 @@ class PreparedStatementWrapper extends StatementWrapper
     public ResultSet executeQuery() throws SQLException {
         try {
             if (this.wrappedStmt != null) {
-                return ((PreparedStatement) this.wrappedStmt).executeQuery();
+            	ResultSet rs = ((PreparedStatement) this.wrappedStmt).executeQuery();
+            	((com.mysql.jdbc.ResultSet)rs).setStatement(this);
+            	
+            	return rs;
             } else {
                 throw new SQLException("No operations allowed after statement closed",
                     SQLError.SQL_STATE_GENERAL_ERROR);
