@@ -33,16 +33,22 @@ import java.io.File;
 import java.io.FileWriter;
 import java.io.Writer;
 
+import java.sql.BatchUpdateException;
 import java.sql.Blob;
 import java.sql.Clob;
 import java.sql.Connection;
+import java.sql.DatabaseMetaData;
+import java.sql.DriverManager;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
+import java.sql.Time;
 import java.sql.Timestamp;
+import java.text.SimpleDateFormat;
 
 import java.util.Calendar;
+import java.util.Date;
 import java.util.Properties;
 import java.util.TimeZone;
 
@@ -149,38 +155,40 @@ public class StatementRegressionTest extends BaseTestCase {
      * @throws Exception if the test fails.
      */
     public void testBug1933() throws Exception {
-        Connection maxRowsConn = null;
-        PreparedStatement maxRowsPrepStmt = null;
-        Statement maxRowsStmt = null;
-
-        try {
-            Properties props = new Properties();
-
-            props.setProperty("maxRows", "1");
-
-            maxRowsConn = getConnectionWithProps(props);
-
-            maxRowsStmt = maxRowsConn.createStatement();
-
-            assertTrue(maxRowsStmt.getMaxRows() == 1);
-
-            this.rs = maxRowsStmt.executeQuery("SELECT 1 UNION SELECT 2");
-
-            this.rs.next();
-
-            maxRowsPrepStmt = maxRowsConn.prepareStatement(
-                    "SELECT 1 UNION SELECT 2");
-
-            assertTrue(maxRowsPrepStmt.getMaxRows() == 1);
-
-            this.rs = maxRowsPrepStmt.executeQuery();
-
-            this.rs.next();
-
-            assertTrue(!this.rs.next());
-        } finally {
-            maxRowsConn.close();
-        }
+    	if (versionMeetsMinimum(4, 0)) {
+	        Connection maxRowsConn = null;
+	        PreparedStatement maxRowsPrepStmt = null;
+	        Statement maxRowsStmt = null;
+	
+	        try {
+	            Properties props = new Properties();
+	
+	            props.setProperty("maxRows", "1");
+	
+	            maxRowsConn = getConnectionWithProps(props);
+	
+	            maxRowsStmt = maxRowsConn.createStatement();
+	
+	            assertTrue(maxRowsStmt.getMaxRows() == 1);
+	
+	            this.rs = maxRowsStmt.executeQuery("SELECT 1 UNION SELECT 2");
+	
+	            this.rs.next();
+	
+	            maxRowsPrepStmt = maxRowsConn.prepareStatement(
+	                    "SELECT 1 UNION SELECT 2");
+	
+	            assertTrue(maxRowsPrepStmt.getMaxRows() == 1);
+	
+	            this.rs = maxRowsPrepStmt.executeQuery();
+	
+	            this.rs.next();
+	
+	            assertTrue(!this.rs.next());
+	        } finally {
+	            maxRowsConn.close();
+	        }
+    	}
     }
 
     /**
@@ -312,129 +320,136 @@ public class StatementRegressionTest extends BaseTestCase {
     public void testBug3620() throws SQLException {
         long epsillon = 3000; // 3 seconds time difference
 
-        try {
-            this.stmt.executeUpdate("DROP TABLE IF EXISTS testBug3620");
-            this.stmt.executeUpdate(
-                "CREATE TABLE testBug3620 (field1 TIMESTAMP)");
-
-            PreparedStatement tsPstmt = this.conn.prepareStatement(
-                    "INSERT INTO testBug3620 VALUES (?)");
-
-            Calendar pointInTime = Calendar.getInstance();
-            pointInTime.set(2004, 02, 29, 10, 0, 0);
-            pointInTime.clear(Calendar.MILLISECOND);
-
-            long pointInTimeOffset = pointInTime.getTimeZone().getRawOffset();
-
-            java.sql.Timestamp ts = new java.sql.Timestamp(pointInTime.getTime()
-                                                                      .getTime());
-
-            tsPstmt.setTimestamp(1, ts);
-            tsPstmt.executeUpdate();
-
-            String tsValueAsString = getSingleValue("testBug3620", "field1",
-                    null).toString();
-
-            System.out.println("Timestamp as string with no calendar: "
-                + tsValueAsString.toString());
-
-            Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
-
-            this.stmt.executeUpdate("DELETE FROM testBug3620");
-
-            Properties props = new Properties();
-            props.put("useTimezone", "true");
-
-            Connection tzConn = getConnectionWithProps(props);
-
-            Statement tsStmt = tzConn.createStatement();
-
-            tsPstmt = tzConn.prepareStatement(
-                    "INSERT INTO testBug3620 VALUES (?)");
-
-            tsPstmt.setTimestamp(1, ts, cal);
-            tsPstmt.executeUpdate();
-
-            tsValueAsString = getSingleValue("testBug3620", "field1", null)
-                                  .toString();
-
-            Timestamp tsValueAsTimestamp = (Timestamp) getSingleValue("testBug3620",
-                    "field1", null);
-
-            System.out.println("Timestamp as string with UTC calendar: "
-                + tsValueAsString.toString());
-            System.out.println("Timestamp as Timestamp with UTC calendar: "
-                + tsValueAsTimestamp);
-
-            this.rs = tsStmt.executeQuery("SELECT field1 FROM testBug3620");
-            this.rs.next();
-
-            Timestamp tsValueUTC = this.rs.getTimestamp(1, cal);
-            
-            //
-            // We use this testcase with other vendors, JDBC spec
-            // requires result set fields can only be read once,
-            // although MySQL doesn't require this ;)
-            //
-            this.rs = tsStmt.executeQuery("SELECT field1 FROM testBug3620");
-            this.rs.next();
-            
-            Timestamp tsValueStmtNoCal = this.rs.getTimestamp(1);
-            
-            System.out.println(
-                "Timestamp specifying UTC calendar from normal statement: "
-                + tsValueUTC.toString());
-
-            PreparedStatement tsPstmtRetr = tzConn.prepareStatement(
-                    "SELECT field1 FROM testBug3620");
-
-            this.rs = tsPstmtRetr.executeQuery();
-            this.rs.next();
-
-            Timestamp tsValuePstmtUTC = this.rs.getTimestamp(1, cal);
-
-            System.out.println(
-                "Timestamp specifying UTC calendar from prepared statement: "
-                + tsValuePstmtUTC.toString());
-
-            //
-            // We use this testcase with other vendors, JDBC spec
-            // requires result set fields can only be read once,
-            // although MySQL doesn't require this ;)
-            //
-            
-            this.rs = tsPstmtRetr.executeQuery();
-            this.rs.next();
-            
-            Timestamp tsValuePstmtNoCal = this.rs.getTimestamp(1);
-
-            
-            System.out.println(
-                "Timestamp specifying no calendar from prepared statement: "
-                + tsValuePstmtNoCal.toString());
-
-            long stmtDeltaTWithCal = (ts.getTime() - tsValueStmtNoCal.getTime());
-
-            assertTrue(
-                "Difference between original timestamp and timestamp retrieved using java.sql.Statement "
-            		+ "set in database using UTC calendar is not ~= "
-                + pointInTimeOffset,
-                (Math.abs(stmtDeltaTWithCal - pointInTimeOffset) < epsillon));
-            
-            long pStmtDeltaTWithCal = (ts.getTime() - tsValuePstmtNoCal.getTime());
-
-            assertTrue(
-                "Difference between original timestamp and timestamp retrieved using java.sql.PreparedStatement "
-            		+ "set in database using UTC calendar is not ~= "
-                + pointInTimeOffset,
-                (Math.abs(pStmtDeltaTWithCal - pointInTimeOffset) < epsillon));
-            
-            System.out.println(
-                "Difference between original ts and ts with no calendar: "
-                + (ts.getTime() - tsValuePstmtNoCal.getTime())
-                + ", offset should be " + pointInTimeOffset);
-        } finally {
-            this.stmt.executeUpdate("DROP TABLE IF EXISTS testBug3620");
+        // This test doesn't work if timezone on server is UTC 
+        // FIXME: Use 'rollForward' flag in C/J 3.1 and newer
+        
+        DatabaseMetaData dbmd = this.conn.getMetaData();
+        
+        if (dbmd.getDriverMajorVersion() >= 3 && dbmd.getDriverMinorVersion() >= 1) {
+	        try {
+	            this.stmt.executeUpdate("DROP TABLE IF EXISTS testBug3620");
+	            this.stmt.executeUpdate(
+	                "CREATE TABLE testBug3620 (field1 TIMESTAMP)");
+	
+	            PreparedStatement tsPstmt = this.conn.prepareStatement(
+	                    "INSERT INTO testBug3620 VALUES (?)");
+	
+	            Calendar pointInTime = Calendar.getInstance();
+	            pointInTime.set(2004, 02, 29, 10, 0, 0);
+	            pointInTime.clear(Calendar.MILLISECOND);
+	
+	            long pointInTimeOffset = pointInTime.getTimeZone().getRawOffset();
+	
+	            java.sql.Timestamp ts = new java.sql.Timestamp(pointInTime.getTime()
+	                                                                      .getTime());
+	
+	            tsPstmt.setTimestamp(1, ts);
+	            tsPstmt.executeUpdate();
+	
+	            String tsValueAsString = getSingleValue("testBug3620", "field1",
+	                    null).toString();
+	
+	            System.out.println("Timestamp as string with no calendar (in database, set via setTimestamp(1, ts) ): "
+	                + tsValueAsString.toString());
+	
+	            Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
+	
+	            this.stmt.executeUpdate("DELETE FROM testBug3620");
+	
+	            Properties props = new Properties();
+	            props.put("useTimezone", "true");
+	
+	            Connection tzConn = getConnectionWithProps(props);
+	
+	            Statement tsStmt = tzConn.createStatement();
+	
+	            tsPstmt = tzConn.prepareStatement(
+	                    "INSERT INTO testBug3620 VALUES (?)");
+	
+	            tsPstmt.setTimestamp(1, ts, cal);
+	            tsPstmt.executeUpdate();
+	
+	            tsValueAsString = getSingleValue("testBug3620", "field1", null)
+	                                  .toString();
+	
+	            Timestamp tsValueAsTimestamp = (Timestamp) getSingleValue("testBug3620",
+	                    "field1", null);
+	
+	            System.out.println("Timestamp as string with UTC calendar (in database, set via setTimestamp(1, ts, cal) ): "
+	                + tsValueAsString.toString());
+	            System.out.println("Timestamp as Timestamp with UTC calendar (in database, set via setTimestamp(1, ts, cal) ): "
+	                + tsValueAsTimestamp);
+	
+	            this.rs = tsStmt.executeQuery("SELECT field1 FROM testBug3620");
+	            this.rs.next();
+	
+	            Timestamp tsValueUTC = this.rs.getTimestamp(1, cal);
+	            
+	            //
+	            // We use this testcase with other vendors, JDBC spec
+	            // requires result set fields can only be read once,
+	            // although MySQL doesn't require this ;)
+	            //
+	            this.rs = tsStmt.executeQuery("SELECT field1 FROM testBug3620");
+	            this.rs.next();
+	            
+	            Timestamp tsValueStmtNoCal = this.rs.getTimestamp(1);
+	            
+	            System.out.println(
+	                "Timestamp specifying UTC calendar from normal statement: "
+	                + tsValueUTC.toString());
+	
+	            PreparedStatement tsPstmtRetr = tzConn.prepareStatement(
+	                    "SELECT field1 FROM testBug3620");
+	
+	            this.rs = tsPstmtRetr.executeQuery();
+	            this.rs.next();
+	
+	            Timestamp tsValuePstmtUTC = this.rs.getTimestamp(1, cal);
+	
+	            System.out.println(
+	                "Timestamp specifying UTC calendar from prepared statement: "
+	                + tsValuePstmtUTC.toString());
+	
+	            //
+	            // We use this testcase with other vendors, JDBC spec
+	            // requires result set fields can only be read once,
+	            // although MySQL doesn't require this ;)
+	            //
+	            
+	            this.rs = tsPstmtRetr.executeQuery();
+	            this.rs.next();
+	            
+	            Timestamp tsValuePstmtNoCal = this.rs.getTimestamp(1);
+	
+	            
+	            System.out.println(
+	                "Timestamp specifying no calendar from prepared statement: "
+	                + tsValuePstmtNoCal.toString());
+	
+	            long stmtDeltaTWithCal = (ts.getTime() - tsValueStmtNoCal.getTime());
+	
+	            assertTrue(
+	                "Difference between original timestamp and timestamp retrieved using java.sql.Statement "
+	            		+ "set in database using UTC calendar is not ~= "
+	                + pointInTimeOffset,
+	                (Math.abs(stmtDeltaTWithCal - pointInTimeOffset) < epsillon));
+	            
+	            long pStmtDeltaTWithCal = (ts.getTime() - tsValuePstmtNoCal.getTime());
+	
+	            assertTrue(
+	                "Difference between original timestamp and timestamp retrieved using java.sql.PreparedStatement "
+	            		+ "set in database using UTC calendar is not ~= "
+	                + pointInTimeOffset,
+	                (Math.abs(pStmtDeltaTWithCal - pointInTimeOffset) < epsillon));
+	            
+	            System.out.println(
+	                "Difference between original ts and ts with no calendar: "
+	                + (ts.getTime() - tsValuePstmtNoCal.getTime())
+	                + ", offset should be " + pointInTimeOffset);
+	        } finally {
+	            this.stmt.executeUpdate("DROP TABLE IF EXISTS testBug3620");
+	        }
         }
     }
 
@@ -498,6 +513,87 @@ public class StatementRegressionTest extends BaseTestCase {
     	assertTrue(output.indexOf(query) != -1);	
     }
     
+    /**
+     * Tests fix for BUG#5874, timezone correction goes in wrong 'direction'
+     * when useTimezone=true and server timezone differs from client timezone.
+     * 
+     * @throws Exception if the test fails.
+     */
+    public void testBug5874() throws Exception {
+    	try {
+    		String clientTimezoneName = "America/Los_Angeles";
+    		String serverTimezoneName = "America/Chicago";
+    		
+    		
+    		TimeZone.setDefault(TimeZone.getTimeZone(clientTimezoneName));
+    		
+    		long epsillon = 3000; // 3 seconds difference
+    		
+    		long clientTimezoneOffsetMillis = TimeZone.getDefault().getRawOffset();
+    		long serverTimezoneOffsetMillis = TimeZone.getTimeZone(serverTimezoneName).getRawOffset();
+    		
+    		long offsetDifference = clientTimezoneOffsetMillis - serverTimezoneOffsetMillis;
+    		
+    		Properties props = new Properties();
+    		props.put("useTimezone", "true");
+    		props.put("serverTimezone", serverTimezoneName);
+    		
+    		Connection tzConn = getConnectionWithProps(props);
+    		Statement tzStmt = tzConn.createStatement();
+    		tzStmt.executeUpdate("DROP TABLE IF EXISTS timeTest");
+    		tzStmt.executeUpdate(
+    		"CREATE TABLE timeTest (tstamp DATETIME, t TIME)");
+    		
+    		PreparedStatement pstmt = tzConn.prepareStatement(
+    		"INSERT INTO timeTest VALUES (?, ?)");
+    		
+    		long now = System.currentTimeMillis(); // Time in milliseconds since 1/1/1970 GMT
+    		
+    		Timestamp nowTstamp = new Timestamp(now);
+    		Time nowTime = new Time(now);
+    		
+    		pstmt.setTimestamp(1, nowTstamp);
+    		pstmt.setTime(2, nowTime);
+    		pstmt.executeUpdate();
+    		
+    		rs = tzStmt.executeQuery("SELECT * from timeTest");
+    		
+    		// Timestamps look like this: 2004-11-29 13:43:21
+    		SimpleDateFormat timestampFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
+    		SimpleDateFormat timeFormat = new SimpleDateFormat("HH:mm:ss");
+    		
+    		while (rs.next()) {
+    			String retrTimestampString = rs.getString(1);
+    			Timestamp retrTimestamp = rs.getTimestamp(1);
+    			
+    			Date timestampOnServer = timestampFormat.parse(retrTimestampString);
+    			
+    			long retrievedOffsetForTimestamp = retrTimestamp.getTime() - timestampOnServer.getTime();
+    			
+    			assertTrue(
+    					"Difference between original timestamp and timestamp retrieved using client timezone is not "
+    					+ offsetDifference,
+						(Math.abs(retrievedOffsetForTimestamp - offsetDifference) < epsillon));
+    			
+    			String retrTimeString = rs.getString(2);
+    			Time retrTime = rs.getTime(2);
+    			
+    			Date timeOnServerAsDate = timeFormat.parse(retrTimeString);
+    			Time timeOnServer = new Time(timeOnServerAsDate.getTime());
+    			
+    			long retrievedOffsetForTime = retrTime.getTime() - timeOnServer.getTime();
+    			
+    			assertTrue(
+    					"Difference between original times and time retrieved using client timezone is not "
+    					+ offsetDifference,
+						(Math.abs(retrievedOffsetForTime - offsetDifference) < epsillon));
+    		}
+    	} finally {
+    		stmt.executeUpdate("DROP TABLE IF EXISTS timeTest");
+    	}
+    }
+    
+    
     /*
     public void testBug5873() throws Exception {
     	try {
@@ -1042,4 +1138,127 @@ public class StatementRegressionTest extends BaseTestCase {
     private void dropGGKTables() throws Exception {
         this.stmt.executeUpdate("DROP TABLE IF EXISTS testggk");
     }
+
+    public void testBug6823() throws SQLException {
+        innerBug6823(true);
+        innerBug6823(false);
+    }
+
+    public void testBug8064() throws SQLException {
+    	// this only works with SJIS, and is only required on 4.0 or older
+    	if (!versionMeetsMinimum(4, 1)) {
+    		String charset = getMysqlVariable("character_set");
+    		
+    		if ("SJIS".equalsIgnoreCase(charset)) {
+		    	try {
+		    		this.stmt.executeUpdate("DROP TABLE IF EXISTS testBug8064");
+		    		this.stmt.executeUpdate("CREATE TABLE testBug8064(bdata BLOB)");
+		    		
+		    		PreparedStatement pStmt = this.conn.prepareStatement("INSERT INTO testBug8064 VALUES (?)");
+		    		byte[] byteData = new byte[] { (byte)0x81, (byte)0x27};
+		    		pStmt.setBytes(1, byteData);
+		    		pStmt.executeUpdate();
+		    	} finally {
+		    		this.stmt.executeUpdate("DROP TABLE IF EXISTS testBug8064");
+		    	}
+    		} else {
+    			warn("Not running testBug8064, requires 4.0 server with 'sjis' as character set");
+    		}
+    	}
+    }
+    
+    /**
+     * @param continueBatchOnError
+     * @throws SQLException
+     */
+    private void innerBug6823(boolean continueBatchOnError) throws SQLException {
+        Properties continueBatchOnErrorProps = new Properties();
+        continueBatchOnErrorProps.setProperty("continueBatchOnError", Boolean
+                .toString(continueBatchOnError));
+        this.conn = getConnectionWithProps(continueBatchOnErrorProps);
+        Statement statement = conn.createStatement();
+
+        String tableName = "testBug6823";
+        try {
+            createTable(tableName,
+                    "(id int not null primary key auto_increment,"
+                            + " strdata1 varchar(255) not null,"
+                            + " strdata2 varchar(255),"
+                            + " UNIQUE INDEX (strdata1))");
+
+            PreparedStatement pStmt = this.conn.prepareStatement("INSERT INTO "
+                    + tableName + " (strdata1, strdata2) VALUES (?,?)");
+
+            int c = 0;
+            addBatchItems(statement, pStmt, tableName, ++c);
+            addBatchItems(statement, pStmt, tableName, ++c);
+            addBatchItems(statement, pStmt, tableName, ++c);
+            addBatchItems(statement, pStmt, tableName, c); // duplicate entry
+            addBatchItems(statement, pStmt, tableName, ++c);
+            addBatchItems(statement, pStmt, tableName, ++c);
+
+            int expectedUpdateCounts = continueBatchOnError ? 6 : 3;
+
+            BatchUpdateException e1 = null;
+            BatchUpdateException e2 = null;
+
+            int[] updateCountsPstmt = null;
+            try {
+                updateCountsPstmt = pStmt.executeBatch();
+            } catch (BatchUpdateException e) {
+                e1 = e;
+                updateCountsPstmt = e1.getUpdateCounts();
+            }
+
+            int[] updateCountsStmt = null;
+            try {
+                updateCountsStmt = statement.executeBatch();
+            } catch (BatchUpdateException e) {
+                e2 = e;
+                updateCountsStmt = e1.getUpdateCounts();
+            }
+
+            assertNotNull(e1);
+            assertNotNull(e2);
+
+            assertEquals(expectedUpdateCounts, updateCountsPstmt.length);
+            assertEquals(expectedUpdateCounts, updateCountsStmt.length);
+
+            if (continueBatchOnError) {
+                assertTrue(updateCountsPstmt[3] == Statement.EXECUTE_FAILED);
+                assertTrue(updateCountsStmt[3] == Statement.EXECUTE_FAILED);
+            }
+
+            int psRows = 0;
+            this.rs = this.stmt.executeQuery("SELECT * from " + tableName
+                    + " WHERE strdata1 like \"ps_%\"");
+            while (rs.next()) {
+                psRows++;
+            }
+            assertTrue(psRows > 0);
+
+            int sRows = 0;
+            this.rs = this.stmt.executeQuery("SELECT * from " + tableName
+                    + " WHERE strdata1 like \"s_%\"");
+            while (rs.next()) {
+                sRows++;
+            }
+            assertTrue(sRows > 0);
+
+            assertTrue(psRows + "!=" + sRows, psRows == sRows);
+        } finally {
+            dropTable(tableName);
+        }
+    }
+
+    private void addBatchItems(Statement statement, PreparedStatement pStmt,
+            String tableName, int i) throws SQLException {
+        pStmt.setString(1, "ps_batch_" + i);
+        pStmt.setString(2, "ps_batch_" + i);
+        pStmt.addBatch();
+
+        statement.addBatch("INSERT INTO " + tableName
+                + " (strdata1, strdata2) VALUES " + "(\"s_batch_" + i
+                + "\",\"s_batch_" + i + "\")");
+    }
 }
